@@ -1,10 +1,10 @@
 #! /usr/bin/env tclsh
 
 package require starkit
-package require tclrivet
-
 starkit::startup
 
+package require tclrivet
+
 proc call_page {} {
 	upvar ::env env
 	set outchan stdout
@@ -102,9 +102,27 @@ proc call_page {} {
 	switch -glob -- [string tolower $targetfile] {
 		"*.rvt" {
 			cd [file dirname $targetfile]
+
+			set env(SCRIPT_FILENAME) $targetfile
+			if {[info exists env(REQUEST_URI)]} {
+				set scriptfilenamework [lrange [file split $targetfile] 1 end]
+				set requesturiwork [lindex [split $env(REQUEST_URI) ?] 0]
+				set requesturiwork [split [string trim $requesturiwork /] /]
+				if {[llength $requesturiwork] == 0} {
+					set requesturiwork [list index.rvt]
+				}
+				for {set sfidx 0} {$sfidx < [llength $scriptfilenamework]} {incr sfidx} {
+					set endidx [expr $sfidx + [llength $requesturiwork] - 1]
+					set chklist [lrange $scriptfilenamework $sfidx $endidx]
+					if {$chklist == $requesturiwork} {
+						set env(SCRIPT_NAME) "/[join [lrange $scriptfilenamework $sfidx end] /]"
+						break
+					}
+				}
+			}
 	
 			if {[catch {
-				parse $targetfile	
+				parse $targetfile
 			} err]} {
 				rivet_error
 				rivet_flush
@@ -639,12 +657,15 @@ proc call_page {} {
 }
 
 proc print_help {} {
-	tcl_puts "Usage: [file tail [info nameofexecutable]] {--server \[--address <address>\] \[--port <port>\]"
-	tcl_puts "       \[--foreground {yes|no}\] \[--init <scp>\] \[--logfile {-|<file>}\] \[--errorlog {-|<file>}\]"
-	tcl_puts "       |--cgi|--help|--version}"
+	tcl_puts "Usage: [file tail [info nameofexecutable]] {--server \[--address <address>\]"
+	tcl_puts "       \[--port <port>\] \[--foreground {yes|no}\] \[--init <scp>\]"
+	tcl_puts "       \[--logfile {-|<file>}\] \[--errorlog {-|<file>}\]|--cgi|--help"
+	tcl_puts "       |--version}"
 	tcl_puts "   --server           Run in standalone server mode"
-	tcl_puts "   --address address  Listen on address for HTTP requests (server mode, default is \"ALL\")"
-	tcl_puts "   --port portno      Listen on port for HTTP requests (server mode, default is \"80\")"
+	tcl_puts "   --address address  Listen on address for HTTP requests (server mode, default"
+	tcl_puts "                      is \"ALL\")"
+	tcl_puts "   --port portno      Listen on port for HTTP requests (server mode, default is"
+	tcl_puts "                      \"80\")"
 	tcl_puts "   --foreground fg    Run in foreground (server mode, default is \"no\")"
 	tcl_puts "   --init script      Run script prior to accepting connections (server mode)"
 	tcl_puts "   --logfile file     Log request information to file (or stdout) (server mode)"
@@ -695,7 +716,6 @@ proc rivet_cgi_server {addr port foreground initscp logfile errorlogfile} {
 		}
 	}
 
-
 	if {!$foreground} {
 		set mypid [fork]
 		if {$mypid != 0} {
@@ -738,6 +758,11 @@ proc rivet_cgi_server_request {hostport logfd elogfd sock addr port} {
 		flush $elogfd
 	}
 
+	# Reap any dead children
+	catch {
+		wait -nohang
+	}
+
 	# Fork off a child to handle the request
 	set mypid [fork]
 	if {$mypid != 0} {
@@ -789,6 +814,7 @@ proc rivet_cgi_server_request_data {sock addr hostport logfd elogfd} {
 			set sockinfo(state) HEADERS
 
 			set sockinfo(url) [join [lrange $work 1 end-1] " "]
+			set sockinfo(url) [regsub {http://[^/]*/} $sockinfo(url) {/}]
 
 			set work [split $sockinfo(url) ?]
 			set sockinfo(path) [lindex $work 0]
@@ -826,11 +852,11 @@ proc rivet_cgi_server_request_data {sock addr hostport logfd elogfd} {
 
 		fileevent $sock readable ""
 
+		set localinfo [fconfigure $sock -sockname]
 		if {![info exists headers(CONNECTION)]} {
 			set headers(CONNECTION) "close"
 		}
 		if {![info exists headers(HOST)]} {
-			set localinfo [fconfigure $sock -sockname]
 			set headers(HOST) [lindex $localinfo 1]
 		}
 
@@ -847,7 +873,7 @@ proc rivet_cgi_server_request_data {sock addr hostport logfd elogfd} {
 		if {[info exists sockinfo(query)]} {
 			set myenv(QUERY_STRING) $sockinfo(query)
 		}
-		## Post requests have additional information
+		### Post requests have additional information
 		if {$sockinfo(method) == "POST"} {
 			if {[info exists headers(CONTENT-TYPE)]} {
 				set myenv(CONTENT_TYPE) $headers(CONTENT-TYPE)
@@ -857,6 +883,9 @@ proc rivet_cgi_server_request_data {sock addr hostport logfd elogfd} {
 			}
 		}
 		## Additional variables
+		set myenv(REQUEST_URI) $sockinfo(url)
+		set myenv(SERVER_ADDR) [lindex $localinfo 0]
+		set myenv(DOCUMENT_ROOT) $::starkit::topdir
 		if {[info exists headers(ACCEPT)]} {
 			set myenv(HTTP_ACCEPT) $headers(ACCEPT)
 		}
@@ -867,6 +896,13 @@ proc rivet_cgi_server_request_data {sock addr hostport logfd elogfd} {
 		if {[info exists headers(COOKIE)]} {
 			set myenv(HTTP_COOKIE) $headers(COOKIE)
 		}
+		## Copy some environment variables directly
+		if {[info exists ::env(SERVER_ADMIN)]} {
+			set myenv(SERVER_ADMIN) $::env(SERVER_ADMIN)
+		}
+		if {[info exists ::env(PATH)]} {
+			set myenv(PATH) $::env(PATH)
+		}
 
 		# Add Rivet Interface specification to fake environment, so further
 		# Rivet/CGI knows how to interface
@@ -878,10 +914,10 @@ proc rivet_cgi_server_request_data {sock addr hostport logfd elogfd} {
 
 		# Call "call_page" with the new enivronment
 		if {[catch {
-#			if {$logfd != ""} {
-#				tcl_puts $logfd "($sock/$addr/[pid]) Debug: [array get ::env] ++ headers = [array get headers] ++ ::rivet::header_sent = $::rivet::header_sent"
-#				flush $logfd
-#			}
+			if {$elogfd != ""} {
+				tcl_puts $elogfd "($sock/$addr/[pid]) Debug: [array get ::env] ++ headers = [array get headers]"
+				flush $elogfd
+			}
 			call_page
 
 			if {$logfd != ""} {
@@ -952,7 +988,7 @@ if {![info exists ::env(GATEWAY_INTERFACE)]} {
 			exit 0
 		}
 		"--version" {
-			tcl_puts "RivetStarkit version 0.5.0.1322"
+			tcl_puts "RivetStarkit version 0.5.0.1346"
 			exit 0
 		}
 		default {
